package com.yizhi.practice.application.excel;

import com.alibaba.fastjson.JSONObject;
import com.yizhi.core.application.file.domain.Template;
import com.yizhi.core.application.file.util.ProxyCreateUtil;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.util.CellRangeAddress;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.Proxy;
import java.net.URL;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.List;

/**
 * excel 导入 包含合并单元格
 */
@Component
public class PracticeExcelImporter  {

    private static Integer httpProxyPort;
    private static String httpProxyUrl;
    private static final Logger logger = LoggerFactory.getLogger(PracticeExcelImporter.class);


    public List<List<String>> parse(String fileUrl,Template template) {
        //
        InputStream inputStream = this.read(fileUrl);
        int columnCount = template.getColumns().size();
        Workbook book = null;
        try {
            book = WorkbookFactory.create(inputStream);
        } catch (Exception e) {
            logger.error("[PracticeExcelImporter : parse] excel文件读取发生错误",e);
            throw new RuntimeException("excel文件读取发生错误");
        }
        List<List<String>> lines = new ArrayList<>();
        Sheet sheet = book.getSheetAt(0);
        for (int i = template.getStartLine() - 1; i <= sheet.getLastRowNum(); i++) {
            List<String> rowValues = new ArrayList<>();
            Row row ;
            for (int j = 0; j < columnCount; j++) {
                String cellValue = "";
                boolean mergedRegion = isMergedRegion(sheet, i, j);
                if (mergedRegion){
                    //合并单元格方式获取整行数据
                    cellValue = getMergedRegionValue(sheet, i, j);
                }else {
                    row = sheet.getRow(i);
                    Cell cell = row.getCell(j);
                    if (null != cell){
                        cellValue = getCellValue(cell);
                    }
                }
                rowValues.add(cellValue);
            }
            lines.add(rowValues);
        }
        logger.info("导入场景对话数据解析: {}",JSONObject.toJSONString(lines));
        return lines;
    }

    private InputStream read(String fileUrl) {
        HttpURLConnection conn = null;

        try {
            URL url = new URL(fileUrl);
            Proxy proxy = ProxyCreateUtil.createUrlProxy(httpProxyUrl, httpProxyPort);
            if (proxy != null) {
                conn = (HttpURLConnection)url.openConnection(proxy);
            } else {
                conn = (HttpURLConnection)url.openConnection();
            }
            conn.setRequestProperty("User-Agent", "Mozilla/4.0 (compatible; MSIE 5.0; Windows NT; DigExt)");
            InputStream inputStream = conn.getInputStream();
            return inputStream;
        } catch (Exception var7) {
            logger.error("文件解析失败", var7);

        }
        return null;
    }

    /**
     * 判断指定的单元格是否是合并单元格
     * @param sheet
     * @param row 行下标
     * @param column 列下标
     * @return
     */
    private  boolean isMergedRegion(Sheet sheet,int row ,int column) {
        int sheetMergeCount = sheet.getNumMergedRegions();
        for (int i = 0; i < sheetMergeCount; i++) {
            CellRangeAddress range = sheet.getMergedRegion(i);
            int firstColumn = range.getFirstColumn();
            int lastColumn = range.getLastColumn();
            int firstRow = range.getFirstRow();
            int lastRow = range.getLastRow();
            if(row >= firstRow && row <= lastRow){
                if(column >= firstColumn && column <= lastColumn){
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * 获取合并单元格的值
     */
    public  String getMergedRegionValue(Sheet sheet ,int row , int column){
        int sheetMergeCount = sheet.getNumMergedRegions();
        for(int i = 0 ; i < sheetMergeCount ; i++){
            CellRangeAddress ca = sheet.getMergedRegion(i);
            int firstColumn = ca.getFirstColumn();
            int lastColumn = ca.getLastColumn();
            int firstRow = ca.getFirstRow();
            int lastRow = ca.getLastRow();

            if(row >= firstRow && row <= lastRow){
                if(column >= firstColumn && column <= lastColumn){
                    Row fRow = sheet.getRow(firstRow);
                    Cell fCell = fRow.getCell(firstColumn);
                    return getCellValue(fCell) ;
                }
            }
        }

        return null ;
    }

    /**
     * 获取单元格的值
     */
    public  String getCellValue(Cell cell){
        CellType cellType = cell.getCellTypeEnum();
        String cellValue;
        switch (cellType){
            case STRING:
                cellValue = cell.getStringCellValue();
                break;
            case BOOLEAN:
                cellValue = String.valueOf(cell.getBooleanCellValue());
                break;
            case FORMULA:
                cellValue = String.valueOf(cell.getCellFormula());
                break;
            case NUMERIC:
                cellValue = getDataCellValue(cell);
                break;
            case ERROR:
                cellValue = "非法字符";
                break;
            default:
                cellValue = "";
        }
        return cellValue;
    }


    /**获取数值类型单元格的值*/
    private String getDataCellValue(Cell cell){
        String cellStr = "";
        String formatType = cell.getCellStyle().getDataFormatString();
        if (formatType.indexOf("%") != -1) {// 判断如果含百分号，则认为百分数类型
            int baifenshuxiaoshuwei = 0;
            if(formatType.indexOf(".")!=-1) { // 判断包含小数点
                baifenshuxiaoshuwei = formatType.substring(formatType.indexOf("."), formatType.indexOf("%")).length()-1; // 获取小数位数
                cellStr = "#.";
                for(int weishu=0;weishu<baifenshuxiaoshuwei;weishu++) {
                    cellStr += "0";
                }
                cellStr = new DecimalFormat(cellStr).format(cell.getNumericCellValue()*100) + "%"; // 保留对应位数百分号
                if(cellStr.startsWith(".")) {// 如果以.开头的，前面加0拼接；
                    cellStr = "0" + cellStr;
                }

            }else {
                // 不含小数点，直接四舍五入整数；
                cellStr = Math.round(cell.getNumericCellValue()*100)+"%";
            }
        }else {
            cell.setCellType(CellType.STRING);
            cellStr = cell.getStringCellValue();
        }
        return cellStr;
    }


    /*public static void main(String[] args) throws FileNotFoundException {
        PracticeExcelImporter importer = new PracticeExcelImporter();
        Template template = new Template();
        List<TemplateColumn> columns = new ArrayList<>();
        columns.add(new TemplateColumn("order","序列号",1));
        columns.add(new TemplateColumn("robotNode","机器人节点话术(必填)",2));
        columns.add(new TemplateColumn("studentNode","学员节点话术(必填)",3));
        columns.add(new TemplateColumn("keyword","关键词(必填)",4));
        columns.add(new TemplateColumn("extendWord","扩展词(非必填)",5));
        columns.add(new TemplateColumn("tabooWord","禁忌词",6));
        template.setColumns(columns);
        RequestContext context = new RequestContext();
        context.setAccountId(1314L);
        context.setSiteId(1314L);
        context.setCompanyId(1314L);
        context.setSiteCode("shyz");
        context.setOrgId(1314L);
        String fileUrl = "D:\\bugdoc\\岗前培训对话脚本导入已审核1105\\促成对话导入脚本.xlsx";
        InputStream inputStream = new FileInputStream(fileUrl);
        int columnCount = template.getColumns().size();
        Workbook book = null;
        try {
            book = WorkbookFactory.create(inputStream);
        } catch (Exception e) {
            logger.error("[PracticeExcelImporter : parse] excel文件读取发生错误",e);
            throw new RuntimeException("excel文件读取发生错误");
        }
        List<List<String>> lines = new ArrayList<>();
        Sheet sheet = book.getSheetAt(0);
        for (int i = template.getStartLine() - 1; i <= sheet.getLastRowNum(); i++) {
            List<String> rowValues = new ArrayList<>();
            Row row ;
            for (int j = 0; j < columnCount; j++) {
                String cellValue = "";
                boolean mergedRegion = importer.isMergedRegion(sheet, i, j);
                if (mergedRegion){
                    //合并单元格方式获取整行数据
                    cellValue = importer.getMergedRegionValue(sheet, i, j);
                }else {
                    row = sheet.getRow(i);
                    Cell cell = row.getCell(j);
                    if (null != cell){
                        cellValue = importer.getCellValue(cell);
                    }
                }
                rowValues.add(cellValue);
            }
            lines.add(rowValues);
        }
        logger.info("导入场景对话数据解析: {}",JSONObject.toJSONString(lines));
    }*/
}
