package com.yizhi.practice.application.exception

import com.yizhi.core.application.exception.BizException
import com.yizhi.practice.application.exception.RequestParameter.ERROR_CODE
import com.yizhi.practice.application.exception.RequestParameter.EXCEPTION
import com.yizhi.util.application.domain.BizResponse
import com.yizhi.util.application.enums.i18n.Constants
import org.slf4j.LoggerFactory
import org.springframework.messaging.handler.annotation.support.MethodArgumentNotValidException
import org.springframework.validation.BeanPropertyBindingResult
import org.springframework.web.HttpRequestMethodNotSupportedException
import org.springframework.web.bind.annotation.ControllerAdvice
import org.springframework.web.bind.annotation.ExceptionHandler
import org.springframework.web.bind.annotation.ResponseBody
import javax.servlet.http.HttpServletRequest

/**
 * @Author: Lvjh
 * @Date 2020/02/19 18:32
 */

@ControllerAdvice
open class ApiResponseHandler {

    private val logger = LoggerFactory.getLogger(javaClass)

    @ExceptionHandler(HttpRequestMethodNotSupportedException::class)
    @ResponseBody
    fun requestMethodNotSupportException(e: HttpRequestMethodNotSupportedException, request: HttpServletRequest): BizResponse<Any> {
        logger.error(" requestMethodNotSupportException uri[${request.requestURI}]", e)
        return BizResponse.fail(Constants.MSG_BIZ_FAIL.code.toString(), Constants.MSG_BIZ_FAIL.name)
    }

    @ExceptionHandler(MethodArgumentNotValidException::class)
    @ResponseBody
    fun parameterInvalidException(e: MethodArgumentNotValidException, request: HttpServletRequest): BizResponse<Any> {
        logger.error(" parameterInvalidException uri[${request.requestURI}]", e)
        var msg = Constants.MSG_BIZ_FAIL.name
        try {
            val propertyBindResult: BeanPropertyBindingResult = e.bindingResult as BeanPropertyBindingResult
            if (propertyBindResult.hasErrors()) {
                val sbf = StringBuffer()
                propertyBindResult.allErrors.forEach {
                    sbf.append(it.defaultMessage).append(";")
                }
                msg = sbf.toString()
            }
            return BizResponse.fail(Constants.MSG_BIZ_FAIL.code.toString(), msg)
        } catch (e: Exception) {
            logger.error(" parameterInvalidException url[${request.requestURL}]", e)
            e.printStackTrace()
            return BizResponse.fail(Constants.MSG_BIZ_FAIL.code.toString(), Constants.MSG_BIZ_FAIL.name)
        }
    }

    @ExceptionHandler(Exception::class)
    @ResponseBody
    fun sysException(e: Exception, request: HttpServletRequest): BizResponse<Any> {

        request.setAttribute(EXCEPTION, getStackTrace(e))
        request.setAttribute(ERROR_CODE, Constants.MSG_BIZ_FAIL.code.toString())
        val clientIp = request.getHeader("X-Real-IP")
        logger.error("Exception requestUrl[${request.requestURI},clientIp:$clientIp] ", e)
        e.printStackTrace()
        return BizResponse.fail(Constants.MSG_BIZ_FAIL.code.toString(), Constants.MSG_BIZ_FAIL.name)
    }

    @ExceptionHandler(BizException::class)
    @ResponseBody
    fun bizException(e: BizException, request: HttpServletRequest): BizResponse<Any> {
        request.setAttribute(ERROR_CODE, e.code)
        request.setAttribute(EXCEPTION, getStackTrace(e))
        logger.error("BizException requestUrl[${request.requestURL}] ${e.code}:${e.msg}")
        return BizResponse.fail(e.code, e.msg)
    }

    private fun getStackTrace(e: Exception): String {
        val stringBuffer = StringBuffer(e.toString() + "\n")
        val messages = e.stackTrace
        messages.forEach {
            stringBuffer.append("\t $it \n")
        }
        return stringBuffer.toString()
    }
}