package com.yizhi.practice.application.third.xunfei;

import java.io.IOException;
import java.io.InputStream;
import java.math.BigDecimal;
import java.util.Base64;
import java.util.concurrent.ExecutorService;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.google.gson.Gson;
import com.yizhi.practice.application.enums.PracticeVoiceTypeEnum;
import com.yizhi.practice.application.third.xunfei.param.IEvaluateResult;
import com.yizhi.practice.application.third.xunfei.vo.EvaluateResult;
import com.yizhi.practice.application.third.xunfei.vo.OkCondition;
import com.yizhi.practice.application.third.xunfei.vo.ResponseData;

import lombok.Builder;
import lombok.experimental.Tolerate;
import okhttp3.Response;
import okhttp3.WebSocket;
import okhttp3.WebSocketListener;

/**
 * 
 * @author xiaoyu
 *
 */
@Builder
public class EvaluateTask extends WebSocketListener {
	private static final Logger LOG = LoggerFactory.getLogger(EvaluateTask.class);
	final Base64.Encoder encoder = Base64.getEncoder();//编码
	private final Base64.Decoder decoder = Base64.getDecoder();//解码

	private static final Gson json = new Gson();

	private String appId;
	private String inputText;
	private InputStream inputVoiceStream;
	private OkCondition condition;
	private ExecutorService threadPool;
	private PracticeVoiceTypeEnum voiceType;
	private Integer recognizeFactor;
	private Integer evaluateFactor;
	
    @Tolerate
	public EvaluateTask() {
		super();
	}

	@Override
	public void onOpen(WebSocket webSocket, Response response) {
		super.onOpen(webSocket, response);
		
		this.threadPool.submit(EvaluateRunnable.builder()
				.appId(this.appId)
				.inputText(this.inputText)
				.inputVoiceStream(this.inputVoiceStream)
				.webSocket(webSocket)
				.voiceType(this.voiceType)
				.recognizeFactor(this.recognizeFactor)
				.evaluateFactor(this.evaluateFactor)
				.build());
	}
	
	@Override
	public void onMessage(WebSocket webSocket, String text) {
		super.onMessage(webSocket, text);
		ResponseData resp = json.fromJson(text, ResponseData.class);
		if (resp != null) {
			if (resp.getCode() != 0) {
				LOG.info( "code=>" + resp.getCode() + " error=>" + resp.getMessage() + " sid=" + resp.getSid());
				closeWebSocket(webSocket);
				return;
			}
			if (resp.getData() != null) {
				if (resp.getData().getStatus() == 2) {
					try {
						String result = new String(decoder.decode(resp.getData().getData()), "UTF-8");
						condition.ok(getResultFromXML(result));
						LOG.trace("sid:{} 最终识别结果 {}", resp.getSid(), result);
					} catch (Exception e) {
						LOG.error("Error occurred:", e);
					} finally {
						closeWebSocket(webSocket);
					}
				}
			}
		}
	}
	
	private IEvaluateResult getResultFromXML(String total) {
		String flucencyString = getStringByRegex(Constants.FLUENCY_REGEX, total);
		String integrityString = getStringByRegex(Constants.INTEGRITY_REGEX, total);
		String phoneString = getStringByRegex(Constants.PHONE_REGEX, total);
		String toneString = getStringByRegex(Constants.TONE_REGEX, total);
		LOG.info("Xun fei - evaluate result: fluency[{}], integrity[{}], phone[{}], tone[{}]",flucencyString, integrityString, phoneString, toneString);
		BigDecimal fluency = BigDecimal.valueOf(Double.valueOf(flucencyString)).divide(BigDecimal.valueOf(100));
		BigDecimal integrity = BigDecimal.valueOf(Double.valueOf(integrityString)).divide(BigDecimal.valueOf(100));
		BigDecimal phone = BigDecimal.valueOf(Double.valueOf(phoneString)).divide(BigDecimal.valueOf(100));
		BigDecimal tone = BigDecimal.valueOf(Double.valueOf(toneString)).divide(BigDecimal.valueOf(100));
		
		return new EvaluateResult(fluency, integrity, phone, tone);
	}

	@Override
	public void onFailure(WebSocket webSocket, Throwable t, Response response) {
		super.onFailure(webSocket, t, response);
		try {
            if (null != response) {
                int code = response.code();
                LOG.info("onFailure code:" + code);
                LOG.info("onFailure body:" + response.body().string());
                if (101 != code) {
                    LOG.info("connection failed");
                }
            }
        } catch (IOException e) {
            LOG.error("Fail when resolve response body:",e);
        } finally {
        	closeWebSocket(webSocket);
        }
	}

	private void closeWebSocket(WebSocket webSocket) {
		webSocket.close(1000, "");
	}
	
	private String getStringByRegex(String regex, String input) {
		String result = null;  
		Pattern pattern = Pattern.compile(regex);  
		Matcher matcher = pattern.matcher(input);  
		matcher.find();
		result = matcher.group(1);
		return result;
	}
}
