package com.yizhi.practice.application.third.xunfei;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.nio.charset.Charset;
import java.text.SimpleDateFormat;
import java.util.Base64;
import java.util.Date;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.TimeZone;
import java.util.concurrent.ExecutorService;

import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;

import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.alibaba.fastjson.JSONObject;
import com.aliyun.oss.model.OSSObject;
import com.google.common.io.Files;
import com.yizhi.practice.application.enums.PracticeVoiceTypeEnum;
import com.yizhi.practice.application.third.aliyun.AliyunUtil;
import com.yizhi.practice.application.third.xunfei.param.IEvaluateResult;
import com.yizhi.practice.application.third.xunfei.vo.OkCondition;

import lombok.Builder;
import okhttp3.HttpUrl;
import okhttp3.OkHttpClient;
import okhttp3.Request;

/**
 * 
 * @author xiaoyu
 *
 */
@Builder
public class XunfeiUtil {
	
	private static final Logger LOG = LoggerFactory.getLogger(XunfeiUtil.class);

	// 语音识别 api url
	private String recognizeUrl;
	
	// 语音合成 api url
	private String synthesisUrl;
	
	// 语音评测 api url
	private String evaluateUrl;
	
	private String appId;
	
	private String apiSecret;
	
	private String apiKey;
	
	private AliyunUtil aliyunUtil;
	
	private OkHttpClient okHttpClient;
	
	private ExecutorService threadPool;
	
	private Integer recognizeFactor;
	
	private Integer evaluateFactor;
	
	// websocket最大等待时间（单位：毫秒）
	private Long maxWait;
	
	// 每次等待时间（单位：毫秒）
	private Long waitEachRound;
	
	// private static final SimpleDateFormat PATH_DATE = new SimpleDateFormat("yyyyMMdd");
	
	/**
	 * 文件地址方式 语音识别. 
	 * @param voiceFilePath 待识别的语音的文件地址
	 * @param voiceType 可接受：PCM, MP3
	 * @return
	 * @throws Exception 
	 */
	public String voiceToString(String voiceFilePath, PracticeVoiceTypeEnum voiceType) throws Exception {
		if(StringUtils.isEmpty(voiceFilePath)) {
			LOG.error("Xunfei voice->string, input file is empty.");
			return null;
		}
		
		OSSObject ossObj = aliyunUtil.download(voiceFilePath);
		InputStream ins = ossObj.getObjectContent();
		
		String result = null;
		try {
			result = voiceToString(ins, voiceType);
		} catch (Exception e) {
			LOG.error("Error occurred:", e);
			throw e;
		} finally {
			if(null != ins) {
				ins.close();
			}
			ossObj.close();
		}
		return result;
	}
	
	/**
	 * 流式 语音识别. 
	 * @param fileStream
	 * @param voiceType 可接受：PCM, MP3
	 * @return
	 * @throws Exception
	 */
	public String voiceToString(InputStream fileStream, PracticeVoiceTypeEnum voiceType) throws Exception {
		if(null == fileStream) {
			LOG.error("Xunfei voice->string, input file is empty.");
			return null;
		}
		
		String url = null;
		try {
			url = getAuthUrl(recognizeUrl, apiKey, apiSecret);
			Request request = new Request.Builder().url(url).build();
			OkCondition condition = new OkCondition();
			this.okHttpClient.newWebSocket(request, RecognizeTask.builder()
					.fileStream(fileStream)
					.appId(this.appId)
					.condition(condition)
					.threadPool(this.threadPool)
					.voiceType(voiceType)
					.recognizeFactor(this.recognizeFactor)
					.evaluateFactor(this.evaluateFactor)
					.build());
			waitAndBlock(condition, this.maxWait);
			return condition.getResult();
		} catch (Exception e) {
			LOG.error("Error occurred when getting Auth url of [voice -> string].", e);
			throw e;
		} finally {
			fileStream.close();
		}
	}
	
	/**
	 * 等待websocket返回结果
	 * @param condition
	 * @param maxWait
	 * @throws InterruptedException
	 */
	private void waitAndBlock(OkCondition condition, Long maxWait) throws InterruptedException {
		long maxWaitTime = maxWait;
		while(!condition.isDone() && maxWaitTime > 0) {
			Thread.sleep(this.waitEachRound); 
			maxWaitTime = maxWaitTime - this.waitEachRound;
		}
	}
	
	/**
	 * 语音合成. 可接受：PCM, MP3
	 * @param inputText 要合成的文字
	 * @param robotConfig 发音机器人的配置（包含发音机器人的声音，语速等，practice_config表的robot_params字段）例如："{name:'xiaoyan'}"
	 * @param savePath 合成的语音文件保存在阿里云的目录 例如：yizhi/practice/xx/xx.pcm
	 * @param voiceType 可接受：PCM, MP3
	 * @return
	 * @throws Exception
	 */
	public String stringToVoice(String inputText, String robotConfig, String savePath, PracticeVoiceTypeEnum voiceType) throws Exception {
		if(StringUtils.isEmpty(inputText) || StringUtils.isEmpty(robotConfig) || StringUtils.isEmpty(savePath)) {
			LOG.error("Xunfei string->voice, params are empty.");
			return null;
		}
		
		if(inputText.length() > 2000) {
			LOG.error("Xunfei string->voice, length of input string [{}] > 2000.", inputText);
			return null;
		}
		
		String url = null;
		try {
			url = getAuthUrl(synthesisUrl, apiKey, apiSecret);
		} catch (Exception e) {
			LOG.error("Error occurred when getting Auth url of [string -> voice].", e);
			throw e;
		}
		
		Request request = new Request.Builder().url(url).build();
		OkCondition condition = new OkCondition();
		
		Map<String, Object> robotConfigMap = (Map<String, Object>) JSONObject.parse(robotConfig);
		this.okHttpClient.newWebSocket(request, new SynthesisTask(this.appId, inputText, robotConfigMap, condition, voiceType));
		waitAndBlock(condition, this.maxWait);
		return uploadOss(convertToByte(condition.getResultList()), savePath);
	}
	
	/**
	 * for test
	 * @param inputText
	 * @param robotConfig
	 * @throws Exception
	 */
	
	/*
	 * @Deprecated public void stringToVoice(String inputText, String robotConfig,
	 * PracticeVoiceTypeEnum vt) throws Exception { String url = null; try { url =
	 * getAuthUrl(synthesisUrl, apiKey, apiSecret); } catch (Exception e) {
	 * LOG.error("Error occurred when getting Auth url of [string -> voice].", e);
	 * throw e; }
	 * 
	 * Request request = new Request.Builder().url(url).build(); OkCondition
	 * condition = new OkCondition();
	 * 
	 * Map<String, Object> robotConfigMap = (Map<String,
	 * Object>)JSONObject.parse(robotConfig);
	 * this.okHttpClient.newWebSocket(request, new SynthesisTask(this.appId,
	 * inputText, robotConfigMap, condition, vt));
	 * 
	 * waitAndBlock(condition, this.maxWait);
	 * 
	 * saveFileToLocal(condition.getResultList(), new File("forTest.mp3")); }
	 */
	 
	
	private void saveFileToLocal(List<String> origin, File target) throws IOException {
		byte[] bytes = convertToByte(origin);
		Files.write(bytes, target);
	}
	
	/*
	 * private String createFolderPath(IFolderPathParam param) { StringBuffer path =
	 * new StringBuffer(); path.append(param.getCompanyId()).append("/")
	 * .append(param.getBusiness()).append("/")
	 * .append(param.getSubBusiness()).append("/") .append(PATH_DATE.format(new
	 * Date())).append("/") .append(param.getUserId()).append("/") .append(new
	 * Date().getTime()) .append(".mp3"); return path.toString(); }
	 */
	
	private byte[] convertToByte(List<String> results) {
		byte[] result = null;
		if(null != results && !results.isEmpty()) {
			for(int i = 0 ; i < results.size(); i++) {
				if(null != results.get(i)) {
					if(i == 0) {
						result = Base64.getDecoder().decode(results.get(i));
					}else {
						result = ArrayUtils.addAll(result, Base64.getDecoder().decode(results.get(i)));
					}
				}
			}
		}
		return result;
	}
	
	private String uploadOss(byte[] input, String fileName) throws IOException {
		if(null == input) {
			LOG.error("No results from Xunfei, do not upload to Ali yun.");
			return null;
		}
		
		try(InputStream ins = new ByteArrayInputStream(input)){
			return aliyunUtil.uploadByStream(fileName, ins);
		}
	}
	
	/**
	 * 文件地址方式 语音评测. 可接受：PCM, MP3, WAV
	 * @param inputText 待对比文本
	 * @param inputVoicePath 待对比语音的地址
	 * @param voiceType 可接受：PCM, MP3, WAV
	 * @return
	 * @throws Exception
	 */
	public IEvaluateResult evaluate(String inputText, String inputVoicePath, PracticeVoiceTypeEnum voiceType) throws Exception {
		if(StringUtils.isEmpty(inputText) || StringUtils.isEmpty(inputVoicePath)) {
			LOG.error("Xunfei evaluate, params are empty.");
			return null;
		}
		
		OSSObject ossObj = aliyunUtil.download(inputVoicePath);
		InputStream ins = ossObj.getObjectContent();
		
		IEvaluateResult result = null;
		try {
			result =
					evaluate(inputText, ins, voiceType);
		} catch (Exception e) {
			LOG.error("Error occurred:", e);
			throw e;
		} finally {
			if(null != ins) {
				ins.close();
			}
			ossObj.close();
		}
		return result;
	}
	
	/**
	 * 流式 语音评测. 可接受：PCM, MP3, WAV
	 * @param inputText
	 * @param inputVoiceStream
	 * @param voiceType 可接受：PCM, MP3, WAV
	 * @return
	 * @throws Exception
	 */
	public IEvaluateResult evaluate(String inputText
			, InputStream inputVoiceStream, PracticeVoiceTypeEnum voiceType) throws Exception {
		if(StringUtils.isEmpty(inputText) || null == inputVoiceStream) {
			LOG.error("Xunfei evaluate, params are empty.");
			return null;
		}
		
		String url = null;
		try {
			url = getAuthUrl(evaluateUrl, apiKey, apiSecret);
		} catch (Exception e) {
			LOG.error("Error occurred when getting Auth url of [evaluate].", e);
			throw e;
		}
		
		Request request = new Request.Builder().url(url).build();
		OkCondition condition = new OkCondition();
		this.okHttpClient.newWebSocket(request, EvaluateTask.builder()
				.appId(this.appId)
				.inputText(inputText)
				.inputVoiceStream(inputVoiceStream)
				.condition(condition)
				.threadPool(this.threadPool)
				.voiceType(voiceType)
				.recognizeFactor(this.recognizeFactor)
				.evaluateFactor(this.evaluateFactor)
				.build());
		waitAndBlock(condition, this.maxWait);
		return condition.getEvaluateResult();
	}
	
	/**
	 * 根据不同的业务拼接不同的讯飞API接口，分为 语音识别、语音合成、语音评测
	 * @param hostUrl 
	 * @param key
	 * @param secret
	 * @return
	 * @throws Exception
	 */
	public String getAuthUrl(String hostUrl, String key, String secret) throws Exception {
        URL url = new URL(hostUrl);
        SimpleDateFormat format = new SimpleDateFormat("EEE, dd MMM yyyy HH:mm:ss z", Locale.US);
        format.setTimeZone(TimeZone.getTimeZone("GMT"));
        String date = format.format(new Date());
        StringBuilder builder = new StringBuilder("host: ").append(url.getHost()).append("\n").//
                append("date: ").append(date).append("\n").//
                append("GET ").append(url.getPath()).append(" HTTP/1.1");
        Charset charset = Charset.forName("UTF-8");
        Mac mac = Mac.getInstance("hmacsha256");
        SecretKeySpec spec = new SecretKeySpec(secret.getBytes(charset), "hmacsha256");
        mac.init(spec);
        byte[] hexDigits = mac.doFinal(builder.toString().getBytes(charset));
        String sha = Base64.getEncoder().encodeToString(hexDigits);

        String authorization = String.format("api_key=\"%s\", algorithm=\"%s\", headers=\"%s\", signature=\"%s\"", key, "hmac-sha256", "host date request-line", sha);
        
        HttpUrl httpUrl = HttpUrl.parse("https://" + url.getHost() + url.getPath()).newBuilder().//
                addQueryParameter("authorization", Base64.getEncoder().encodeToString(authorization.getBytes(charset))).//
                addQueryParameter("date", date).//
                addQueryParameter("host", url.getHost()).//
                build();
        
        // http,https 换成 ws,wss
        String wsUrl = httpUrl.toString().replace("http://", "ws://").replace("https://", "wss://");
        return wsUrl;
    }
}
